/*
 *  AdORB
 *  Copyright (C) 2006 Victor Ananiev
 *
 *  This file is part of IDLBrowser.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

#import <Foundation/Foundation.h>
#import <Appkit/Appkit.h>
#import "IDLDefinitionScope_Private.h"
#import "IDLDefinition_Private.h"
#import "ObjectCell.h"
#import "IDLViewer.h"

@implementation IDLDefinitionScope(Viewer)

- (NSString*) category
{
    return @"- Modules -";
}

- (NSImage*) cellImage
{
	return [NSImage imageNamed:@"module13"];
}

- (void) loadCell:cell
{
    [cell setRepresentedObject:self];
    [cell setLoaded:YES];
	[cell setImage:[self cellImage]];
    [cell setLeaf:NO];
}

- (NSString*) title
{
    return [self name];
}

- (NSString*) stringValue
{
    if([self name]){
        if([self prefix])
            return [NSString stringWithFormat:@"module %@ <%@>",
                [self name], [self prefix]];
        else
            return [NSString stringWithFormat:@"module %@",[self name]];
    }else
        return @"Root Scope";
}

- (NSArray*) modules
{
	NSEnumerator* en = [_definitionDict objectEnumerator];
	id obj;
	NSMutableArray* array = [NSMutableArray array];
	while(obj = [en nextObject]){
		if([obj isKindOfClass:[IDLDefinitionScope class]])
			[array addObject:obj];
	}
	NSSortDescriptor* desc = [[[NSSortDescriptor alloc]initWithKey:@"name" ascending:YES]autorelease];
	[array sortUsingDescriptors:[NSArray arrayWithObject:desc]];
	return [NSArray arrayWithArray:array];
}

- (NSDictionary*) _getCategories
{
    NSEnumerator* en = [_definitionDict keyEnumerator];
    NSString* key;
    NSMutableDictionary* dict = [NSMutableDictionary dictionary];

    while(key = [en nextObject]){
        IDLDefinition* def = [_definitionDict objectForKey:key];
        NSString* category = [def category];
        if(category){
            NSMutableDictionary* cdict;
            if(!(cdict = [dict objectForKey:category]))
                [dict setObject:cdict = [NSMutableDictionary dictionary] forKey:category];
            [cdict setObject:def forKey:key];
        }
    }
    return dict;
}

- (void) fillMatrix:matrix
{
    NSDictionary* dict = [self _getCategories];
    NSArray* categories = [[dict allKeys]sortedArrayUsingSelector:@selector(compare:)];
    NSEnumerator* catEn = [categories objectEnumerator];
    NSString* cat;
    
    while(cat = [catEn nextObject])
        [[dict objectForKey:cat]fillMatrix:matrix];
}

- (BOOL) writeDefinitionToPboard:(NSPasteboard*)pboard
{
    return NO;
}

@end
