/*
 *  AdORB
 *  Copyright (C) 2006 Victor Ananiev
 *
 *  This file is part of IDLBrowser.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

#import <Foundation/Foundation.h>
#import "IDLDefinition_Private.h"
#import "IDLStruct.h"
#import "IDLStructMember.h"
#import "IDLException.h"
#import "IDLObject.h"
#import "ObjectCell.h"
#import "IDLViewer.h"


@implementation IDLStruct(Viewer)

- (NSImage*) cellImage
{
	return [NSImage imageNamed:@"struct13"];
}

- (void) loadCell:cell
{
    [cell setLoaded:YES];
    [cell setLeaf:NO];
	[cell setImage:[self cellImage]];
    [cell setRepresentedObject:self];
}

- (BOOL) shouldSortItems
{
    return ([self sortMatrixItems] && [_members count] > [self sortIfGreaterThan]);
}

- (void) fillMatrix:matrix
{
    NSEnumerator* enumerator;
    IDLStructMember* member;
    int n = 0;
    id cell;

    if([self shouldSortItems]){
		NSSortDescriptor* sd = [[NSSortDescriptor alloc]initWithKey:@"name" 
				ascending:YES selector:@selector(caseInsensitiveCompare:)];
		NSArray* array = [_members sortedArrayUsingDescriptors:[NSArray arrayWithObject:sd]];
        enumerator = [array objectEnumerator];
		[sd release];
    }else
        enumerator = [_members objectEnumerator];
    
    while(member = [enumerator nextObject]){
        IDLDefinition* type = [member type];
		NSString* key = [member name];
        [matrix addRow];
        [cell = [matrix cellAtRow:n++ column:0]setStringValue:key];
        [type loadCell:cell];
    }
}

- (NSString*) title
{
    return [self name];
}

- (NSString*) stringValue
{
    return [NSString stringWithFormat:@"struct %@",[self name]];
}

@end

@implementation IDLException(Viewer)
- (NSString*) stringValue
{
    return [NSString stringWithFormat:@"exception %@",[self name]];
}

- (NSString*) title
{
    return @"exception";
}

- (NSImage*) cellImage
{
	return [NSImage imageNamed:@"exception13"];
}

@end
