/*
 *  AdORB
 *  Copyright (C) 2006 Victor Ananiev
 *
 *  This file is part of IDLBrowser.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

#import "ObjectCell.h"
#import <Foundation/Foundation.h>
#import "IDLDefinition_Private.h"
#import "IDLViewer.h"

@implementation ObjectCell

- object
{
	return [self representedObject];
}

- (void) setObject:object
{
	[self setRepresentedObject:object];
}

NSImage* DraggingImageForAttributedString(NSAttributedString* string)
{
    float rd = 6; 
    NSSize size = [string size];
    NSRect r = {{0,0}, {size.width+12, size.height+6}};
    NSRect r1 = {{6, 3}, {size.width, size.height}};
    NSPoint p0 = {rd+0.5, 0.5};
    NSPoint p[4] = {{r.size.width-rd-0.5, rd+0.5}, {r.size.width-rd-0.5, r.size.height-rd-0.5}, 
        {rd+0.5, r.size.height-rd-0.5}, {rd+0.5, rd+0.5}};
    NSImage* image = [[[NSImage alloc]initWithSize:r.size]autorelease];
    NSBezierPath* path = [NSBezierPath bezierPath];
    [path moveToPoint:p0];
    [path appendBezierPathWithArcWithCenter:p[0] radius:rd startAngle:-90 endAngle:0];
    [path appendBezierPathWithArcWithCenter:p[1] radius:rd startAngle:0 endAngle:90];
    [path appendBezierPathWithArcWithCenter:p[2] radius:rd startAngle:90 endAngle:180];
    [path appendBezierPathWithArcWithCenter:p[3] radius:rd startAngle:180 endAngle:270];
    [path closePath];
    [image lockFocus];
    [[NSColor colorWithCalibratedWhite:0.5 alpha:0.25]set];
    [path fill];
    [[NSColor colorWithCalibratedWhite:0.0 alpha:0.75]set];
    [path setLineWidth:0.25];
    [path stroke];
    [string drawInRect:r1];
    [image unlockFocus];
    return image;
}

NSImage* DraggingImageForString(NSString* string, NSDictionary* attrDict)
{
	return DraggingImageForAttributedString([[[NSAttributedString alloc]initWithString:string attributes:attrDict]autorelease]);
}

- (NSImage*) draggingImage
{
    return DraggingImageForAttributedString([self attributedStringValue]);
}

- (BOOL)trackMouse:(NSEvent *)e0 
        inRect:(NSRect)cellFrame 
        ofView:(NSView *)controlView 
        untilMouseUp:(BOOL)flag;
{
    NSEvent* e = e0;
    NSPoint p0 = [controlView convertPoint:[e0 locationInWindow] fromView:nil];
    NSPoint p = p0;
    
    while([e type] != NSLeftMouseUp && 
                fabs(p.x - p0.x)*4 + fabs(p.y - p0.y) < 4.0 &&
                NSMouseInRect(p, cellFrame, [controlView isFlipped])){
        e = [NSApp nextEventMatchingMask:NSLeftMouseUpMask | NSLeftMouseDraggedMask
                untilDate:[NSDate distantFuture]
                inMode: NSEventTrackingRunLoopMode
                dequeue:YES];
        p = [controlView convertPoint:[e locationInWindow] fromView:nil];
    }
    if([e type] == NSLeftMouseUp)
        return YES;
    else{
        NSPasteboard* pboard = [NSPasteboard pasteboardWithName:NSDragPboard];
		id object = [self representedObject];
        if([object respondsToSelector:@selector(writeDefinitionToPboard:)] && 
                    [object writeDefinitionToPboard:pboard]){
            NSImage* image = [self draggingImage];
            NSSize offset = {p.x - p0.x, p.y - p0.y};
            NSSize s = [image size];
            NSPoint loc = {cellFrame.origin.x /*+ offset.width*/, 
                /*cellFrame.origin.y + cellFrame.size.height + offset.height*/
                p.y + floor(s.height/2)};
            [controlView dragImage:image
                    at:loc
                    offset:offset
                    event:e0
                    pasteboard:pboard
                    source:self
                    slideBack:YES];
        }
    }
    return NO;
    //return [super trackMouse:e inRect:cellFrame ofView:controlView untilMouseUp:flag];
}

- (unsigned int)draggingSourceOperationMaskForLocal:(BOOL)isLocal
{
    if(isLocal)
        return NSDragOperationGeneric;
    else
        return NSDragOperationNone;
}

- (NSAttributedString*) attributedStringValue
{
	NSMutableAttributedString* string = [[[NSMutableAttributedString alloc]initWithString:[self stringValue] 
		attributes:[NSDictionary dictionaryWithObject:[self font] forKey:NSFontAttributeName]]autorelease];
	NSImage* image = [self image];
	if(image){
		NSTextAttachment* attachment = [[[NSTextAttachment alloc]initWithFileWrapper:nil]autorelease];
		NSTextAttachmentCell* cell = [[[NSTextAttachmentCell alloc]initImageCell:image]autorelease];
		[attachment setAttachmentCell:cell];
		unichar c = NSAttachmentCharacter;
		NSDictionary* dict = [NSDictionary dictionaryWithObjectsAndKeys:
				attachment, NSAttachmentAttributeName,
				[NSNumber numberWithFloat:-2.0], NSBaselineOffsetAttributeName, nil];
		NSAttributedString* imageString = [[[NSAttributedString alloc]initWithString:[NSString stringWithCharacters:&c length:1]
				attributes:dict]autorelease];
		[string insertAttributedString:imageString atIndex:0];
	}
	return string;
}

@end

