/*
 *  AdORB
 *  Copyright (C) 2006 Victor Ananiev
 *
 *  This file is part of IDLBrowser.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

#import "IORField.h"
#import "CORBAObject.h"
#import "IDLItem.h"

@interface NSObject(IORFieldDelegate)
- (void) setScope;
- (BOOL) acceptConnections:(NSArray*)connections;
@end

@implementation IORField

extern NSString* CORBAIORPboardType;
extern NSString* CORBAObjectPboardType;

- initWithFrame:(NSRect)rect
{
    NSArray* types = [NSArray arrayWithObjects:CORBAObjectPboardType, CORBAIORPboardType, NSStringPboardType,
        NSFilenamesPboardType, NSURLPboardType,nil];
	self = [super initWithFrame:rect];
	IORFormatter* formatter = [[[IORFormatter alloc]init]autorelease];
	[formatter setFormatMode:1];
	[self setFormatter:formatter];
    [self registerForDraggedTypes:types];
	return self;
}

- initWithCoder:(NSCoder*)coder
{
    NSArray* types = [NSArray arrayWithObjects:CORBAObjectPboardType, CORBAIORPboardType, NSStringPboardType,
        NSFilenamesPboardType, NSURLPboardType,nil];
	self = [super initWithCoder:coder];
	IORFormatter* formatter = [[[IORFormatter alloc]init]autorelease];
	[formatter setFormatMode:1];
	[self setFormatter:formatter];
    [self registerForDraggedTypes:types];
	return self;
}

#pragma mark Dragging

- (void) highlight
{
    BOOL isFocus = [NSView focusView] == self;
    NSRect r = [self bounds];
    if(!isFocus)
        [self lockFocus];
    [[NSColor blackColor]set];
    NSFrameRectWithWidth(r, 2.0);
    [[self window] flushWindow];
    [[NSGraphicsContext currentContext]flushGraphics];
    if(!isFocus)
        [self unlockFocus];
}


- (unsigned int) draggingSourceOperationMaskForLocal:(BOOL)flag
{
    return NSDragOperationGeneric;
}

- (void) mouseDown:(NSEvent*)e
{
    id objConn = [self objectValue];
    if(objConn){
		NSPoint p = [self convertPoint:[e locationInWindow] fromView:nil];
		NSArray* types = [NSArray arrayWithObjects:CORBAObjectPboardType, CORBAIORPboardType, NSStringPboardType, nil];
		id pboard = [NSPasteboard pasteboardWithName:NSDragPboard];
		NSString* ior = [(CORBAObject*)objConn toString];
		id image = [NSImage imageNamed:@"connected"];
		NSSize s = [image size];
		NSString* data = [NSString stringWithFormat:@"0x%x",objConn];
		NSSize off = {0,0};
		[pboard declareTypes:types owner:nil];
		[pboard setString:data forType:CORBAObjectPboardType];
		[pboard setString:ior forType:CORBAIORPboardType];
		[pboard setString:ior forType:NSStringPboardType];
		p.x -= floor(s.width/2);
		p.y += floor(s.height/2); // flipped
		[self dragImage:image at:p offset:off
				event:e
				pasteboard:pboard
				source:self
				slideBack:YES];
    }
}

static unsigned int op = NSDragOperationNone;

- (unsigned int)draggingEntered:(id <NSDraggingInfo>)sender
{
    if([sender draggingSource] != self){
        [self highlight];
        return op = NSDragOperationGeneric;
    }else
        return op = NSDragOperationNone;
}

- (void) draggingExited:(id <NSDraggingInfo>)sender
{
    op = NSDragOperationNone;
    [self setNeedsDisplay:YES];
}

- (unsigned int)draggingUpdated:(id <NSDraggingInfo>)sender
{
    return NSDragOperationGeneric;
}

static NSMutableArray* draggedObjects = nil;
- (BOOL)prepareForDragOperation:(id <NSDraggingInfo>)sender
{
    NSArray* types = [NSArray arrayWithObjects:CORBAIORPboardType, NSStringPboardType,
                        NSFilenamesPboardType, NSURLPboardType, CORBAObjectPboardType, nil];
    NSPasteboard* pboard = [sender draggingPasteboard];
    NSString* type = [pboard availableTypeFromArray:types];
    id draggingSource = [sender draggingSource];
    BOOL accept = NO;
    if(!draggedObjects)
        draggedObjects = [[NSMutableArray alloc]init];
	else
		[draggedObjects removeAllObjects];
    
	 // Set current IDLDefinitionScope
    if([[self delegate] respondsToSelector:@selector(setScope)])
        [[self delegate] setScope];
    
	if([type isEqualToString:CORBAObjectPboardType] && draggingSource){
        id obj = nil;
        if([draggingSource respondsToSelector:@selector(connection)])
            obj = [draggingSource connection];
        if(obj){
            [draggedObjects addObject:obj];
            accept = YES;
        }
    }else
    if([type isEqualToString:CORBAIORPboardType]){
        NSString* ior = [pboard stringForType:CORBAIORPboardType];
        id obj = nil;
        @try{
			obj = [CORBAObject objectWithIOR:ior];
        }@catch(NSException *localException){
			NSBeginAlertSheet(@"Exception", NULL,NULL,NULL,
				[self window], nil, NULL, NULL, NULL,
				@"Invalid CORBA URL - %@: %@", [localException name], [localException reason]);
			obj = nil;
        }
        if(obj){
            [draggedObjects addObject:obj];
            accept = YES;
        }
    }else
    if([type isEqualToString:NSStringPboardType]){
        NSString* ior = [pboard stringForType:NSStringPboardType];
        id obj = nil;
        @try{
			obj = [CORBAObject objectWithURL:ior];
        }@catch(NSException *localException){
			NSBeginAlertSheet(@"Exception", NULL,NULL,NULL,
				[self window], nil, NULL, NULL, NULL,
				@"Invalid CORBA URL - %@: %@", [localException name], [localException reason]);
			obj = nil;
        }
        if(obj){
            [draggedObjects addObject:obj];
            accept = YES;
        }
    }else
    if([type isEqualToString:NSFilenamesPboardType]){
        NSArray* files = [pboard propertyListForType:NSFilenamesPboardType];
        NSEnumerator* en = [files objectEnumerator];
        NSString* file = nil;
        @try{
			while(file = [en nextObject]){
				id obj = nil;
				NSString* ior = [NSString stringWithContentsOfFile:file encoding:NSUTF8StringEncoding error:NULL];
				if(ior)
					obj = [CORBAObject objectWithURL:ior];
				if(obj)
					[draggedObjects addObject:obj];
			}
        }@catch(NSException *localException){
			NSBeginAlertSheet(@"Exception", NULL,NULL,NULL,
				[self window], nil, NULL, NULL, NULL,
				@"File '%@'\ndoes not contain valid CORBA URL or IOR.\n%@: %@",
					file, [localException name], [localException reason]);
        }
        accept = [draggedObjects count] > 0;
    }else
    if([type isEqualToString:NSURLPboardType]){
        NSURL* url = [NSURL URLFromPasteboard:pboard];
        NSString* ior = [NSString stringWithContentsOfURL:url encoding:NSUTF8StringEncoding error:NULL];
        id obj = nil;
        @try{
			if(ior)
				obj = [CORBAObject objectWithURL:ior];
        }@catch(NSException *localException){
			NSBeginAlertSheet(@"Exception", NULL,NULL,NULL,
				[self window], nil, NULL, NULL, NULL,
				@"URL '%@'\ndoes not contain valid CORBA URL or IOR.\n%@: %@",
					url, [localException name], [localException reason]);
			obj = nil;
        }
        if(obj){
            [draggedObjects addObject:obj];
            accept = YES;
        }
    
    }
    if(!accept)
        [self setNeedsDisplay:YES];
    return accept;
}

- (BOOL)performDragOperation:(id <NSDraggingInfo>)sender
{
    BOOL ok = NO;
    if([[self delegate] respondsToSelector:@selector(acceptConnections:)])
        ok = [[self delegate] acceptConnections:draggedObjects];
    [draggedObjects removeAllObjects];
    if(!ok)
        [self setNeedsDisplay:YES];
    return ok;
}


- (void) concludeDragOperation:(id <NSDraggingInfo>)sender
{
    [self setNeedsDisplay:YES];
}

- (BOOL)shouldDelayWindowOrderingForEvent:(NSEvent *)e
{
	return YES;
}

@end
