/*
 *  AdORB
 *  Copyright (C) 2006 Victor Ananiev
 *
 *  This file is part of IDLBrowser.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

#import <Foundation/Foundation.h>
#import "IORView.h"
#import "IDLInterface_Private.h"
#import "CORBAObject.h"

@implementation IORView

NSString* CORBAObjectPboardType = @"CORBAObject pasteboard type";
NSString* CORBAIORPboardType = @"CORBA IOR pasteboard type";
static NSImage* ior = nil;

+ (void) _loadImages
{
    NSBundle* bundle = [NSBundle bundleForClass:[self class]];
    NSArray* names = [NSArray arrayWithObjects:
        @"connected", @"disconnected", @"ior", @"value", nil];
    NSEnumerator* en = [names objectEnumerator];
    NSString* name;
    
    while(name = [en nextObject]){
        NSString* path = [bundle pathForResource:name ofType:@"tiff"];
        NSImage* image = [[NSImage alloc]initWithContentsOfFile:path];
        [image setName:name];
    }
}

+ (void) initialize
{
    if([IORView class] == self){
        [self _loadImages];
        ior = [NSImage imageNamed:@"ior"];
    }
}

- (void) invalidateConnection:(NSNotification*)notification
{
    if([notification object] == _object)
		[self setConnection:nil];
}

- initWithFrame:(NSRect)r
{
    NSArray* types = [NSArray arrayWithObjects:CORBAObjectPboardType, CORBAIORPboardType, NSStringPboardType,
        NSFilenamesPboardType, NSURLPboardType,nil];
    self = [super initWithFrame:r];
    [self setAutoresizingMask:NSViewNotSizable];
    [self registerForDraggedTypes:types];
    [[NSNotificationCenter defaultCenter]addObserver:self
                    selector:@selector(invalidateConnection:)
                    name:ORBInvalidateObject
                    object:nil];
    return self;
}

- (void) dealloc
{
    [[NSNotificationCenter defaultCenter]removeObserver:self];
    [_object release];
    [super dealloc];
}

- (BOOL) isFlipped
{
    return NO;
}

- (BOOL) acceptsFirstMouse:(NSEvent*)e
{
    return YES;
}

- (BOOL)shouldDelayWindowOrderingForEvent:(NSEvent *)theEvent
{
    return YES;
}

- (void) setConnection:conn
{
    if(conn != _object){
        [_object release];
        _object = [conn retain];
        [self setNeedsDisplay:YES];
    }
}

- connection
{
    return _object;
}

- delegate
{
    return delegate;
}

- (void) setDelegate:anObj
{
    delegate = anObj;
}

- (void) drawBackground
{
    NSRect b = [self bounds];
    NSRect clip = [self bounds];
    NSSize s = [ior size];
    NSPoint p;
    NSRectEdge sides[] = {NSMinXEdge, NSMaxYEdge, NSMaxXEdge, NSMinYEdge, 
                    NSMinXEdge, NSMaxYEdge, NSMaxXEdge, NSMinYEdge};
    float grays[] = {NSBlack,NSBlack,NSLightGray,NSLightGray,
                    NSDarkGray, NSDarkGray, 0.5, 0.5};
    b = NSDrawTiledRects(b,clip,sides,grays,4);
    p.x = b.origin.x + floor((b.size.width - s.width)/2);
    p.y = b.origin.y + floor((b.size.height - s.height)/2);
    [ior compositeToPoint:p operation:NSCompositeSourceOver];
}

- (void) drawRect:(NSRect)rect
{
    NSRect r = [self bounds];
    [[NSGraphicsContext currentContext]saveGraphicsState];
    [self drawBackground];
    if([self connection]){
        id image = [NSImage imageNamed:@"connected"];
        NSSize s = [image size];
	r = NSInsetRect(r,2,2);
	NSRectClip(r);
        r.origin.x = r.origin.x + floor((r.size.width - s.width)/2);
        r.origin.y = r.origin.y + floor((r.size.height - s.height)/2);
        [image compositeToPoint:r.origin operation:NSCompositeSourceOver];
    }
    [[NSGraphicsContext currentContext]restoreGraphicsState];
}

- (void) highlight
{
    BOOL isFocus = [NSView focusView] == self;
    NSRect r = [self bounds];
    if(!isFocus)
        [self lockFocus];
    [[NSColor darkGrayColor]set];
    NSFrameRectWithWidth(r, 2.0);
    [[self window] flushWindow];
    [[NSGraphicsContext currentContext]flushGraphics];
    if(!isFocus)
        [self unlockFocus];
}

@end

@implementation IORView(Dragging)

- (unsigned int) draggingSourceOperationMaskForLocal:(BOOL)flag
{
    return NSDragOperationGeneric;
}


- (void) mouseDown:(NSEvent*)e
{
    id objConn = [self connection];
    if(objConn){
        NSArray* types = [NSArray arrayWithObjects:CORBAObjectPboardType, CORBAIORPboardType, NSStringPboardType, nil];
        id pboard = [NSPasteboard pasteboardWithName:NSDragPboard];
        id image = [NSImage imageNamed:@"connected"];
        NSString* ior = [(CORBAObject*)objConn toString];
        NSRect r = [self bounds];
        NSSize off = {0,0};
        NSString* data = [NSString stringWithFormat:@"0x%x",objConn];
        [pboard declareTypes:types owner:nil];
        [pboard setString:data forType:CORBAObjectPboardType];
        [pboard setString:ior forType:CORBAIORPboardType];
        [pboard setString:ior forType:NSStringPboardType];
        r.size = [image size];
        r.origin.x += floor(([self bounds].size.width - r.size.width)/2);
        r.origin.y += floor(([self bounds].size.height - r.size.height)/2);
        [self dragImage:image at:r.origin offset:off
            event:e
            pasteboard:pboard
            source:self
            slideBack:YES];
    }
}

static unsigned int op = NSDragOperationNone;

- (unsigned int)draggingEntered:(id <NSDraggingInfo>)sender
{
    if([sender draggingSource] != self){
        [self highlight];
        return op = NSDragOperationGeneric;
    }else
        return op = NSDragOperationNone;
}

- (void) draggingExited:(id <NSDraggingInfo>)sender
{
    op = NSDragOperationNone;
    [self setNeedsDisplay:YES];
}

- (unsigned int)draggingUpdated:(id <NSDraggingInfo>)sender
{
    return NSDragOperationGeneric;
}

static NSMutableArray* draggedObjects = nil;
- (BOOL)prepareForDragOperation:(id <NSDraggingInfo>)sender
{
    NSArray* types = [NSArray arrayWithObjects:CORBAObjectPboardType, CORBAIORPboardType, NSStringPboardType,
                        NSFilenamesPboardType, NSURLPboardType,nil];
    NSPasteboard* pboard = [sender draggingPasteboard];
    NSString* type = [pboard availableTypeFromArray:types];
    id draggingSource = [sender draggingSource];
    BOOL accept = NO;
    if(!draggedObjects)
        draggedObjects = [[NSMutableArray alloc]init];
	else
		[draggedObjects removeAllObjects];
    
	 // Set current IDLDefinitionScope
    if([delegate respondsToSelector:@selector(setScope)])
        [delegate setScope];
    
	if([type isEqualToString:CORBAObjectPboardType] && draggingSource){
        id obj = nil;
        if([draggingSource respondsToSelector:@selector(connection)])
            obj = [draggingSource connection];
        if(obj){
            [draggedObjects addObject:obj];
            accept = YES;
        }
    }else
    if([type isEqualToString:CORBAIORPboardType]){
        NSString* ior = [pboard stringForType:CORBAIORPboardType];
        id obj = nil;
        @try{
			obj = [CORBAObject objectWithIOR:ior];
        }@catch(NSException *localException){
			NSBeginAlertSheet(@"Exception", NULL,NULL,NULL,
				[self window], nil, NULL, NULL, NULL,
				@"Invalid CORBA URL - %@: %@", [localException name], [localException reason]);
			obj = nil;
        }
        if(obj){
            [draggedObjects addObject:obj];
            accept = YES;
        }
    }else
    if([type isEqualToString:NSStringPboardType]){
        NSString* ior = [pboard stringForType:NSStringPboardType];
        id obj = nil;
        @try{
			obj = [CORBAObject objectWithURL:ior];
        }@catch(NSException *localException){
			NSBeginAlertSheet(@"Exception", NULL,NULL,NULL,
				[self window], nil, NULL, NULL, NULL,
				@"Invalid CORBA URL - %@: %@", [localException name], [localException reason]);
			obj = nil;
        }
        if(obj){
            [draggedObjects addObject:obj];
            accept = YES;
        }
    }else
    if([type isEqualToString:NSFilenamesPboardType]){
        NSArray* files = [pboard propertyListForType:NSFilenamesPboardType];
        NSEnumerator* en = [files objectEnumerator];
        NSString* file = nil;
        @try{
			while(file = [en nextObject]){
				id obj = nil;
				NSString* ior = [NSString stringWithContentsOfFile:file encoding:NSUTF8StringEncoding error:NULL];
				if(ior)
					obj = [CORBAObject objectWithURL:ior];
				if(obj)
					[draggedObjects addObject:obj];
			}
        }@catch(NSException *localException){
			NSBeginAlertSheet(@"Exception", NULL,NULL,NULL,
				[self window], nil, NULL, NULL, NULL,
				@"File '%@'\ndoes not contain valid CORBA URL or IOR.\n%@: %@",
					file, [localException name], [localException reason]);
        }
        accept = [draggedObjects count] > 0;
    }else
    if([type isEqualToString:NSURLPboardType]){
        NSURL* url = [NSURL URLFromPasteboard:pboard];
        NSString* ior = [NSString stringWithContentsOfURL:url encoding:NSUTF8StringEncoding error:NULL];
        id obj = nil;
        @try{
			if(ior)
				obj = [CORBAObject objectWithURL:ior];
        }@catch(NSException *localException){
			NSBeginAlertSheet(@"Exception", NULL,NULL,NULL,
				[self window], nil, NULL, NULL, NULL,
				@"URL '%@'\ndoes not contain valid CORBA URL or IOR.\n%@: %@",
					url, [localException name], [localException reason]);
			obj = nil;
        }
        if(obj){
            [draggedObjects addObject:obj];
            accept = YES;
        }
    
    }
    if(!accept)
        [self setNeedsDisplay:YES];
    return accept;
}

- (BOOL)performDragOperation:(id <NSDraggingInfo>)sender
{
    BOOL ok = NO;
    if([delegate respondsToSelector:@selector(acceptConnections:)])
        ok = [delegate acceptConnections:draggedObjects];
    [draggedObjects removeAllObjects];
    if(!ok)
        [self setNeedsDisplay:YES];
    return ok;
}


- (void) concludeDragOperation:(id <NSDraggingInfo>)sender
{
    [self setNeedsDisplay:YES];
}

@end

@interface MTObjectView:IORView
{
}
@end
@implementation MTObjectView
// Need to rename this to IORView in IB
@end
