/*
 *  AdORB
 *  Copyright (C) 2006 Victor Ananiev
 *
 *  This file is part of ADORB.
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Library General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Library General Public License for more details.
 *
 *  You should have received a copy of the GNU Library General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#import "HelloWorld.h"
#import <ADORB/ADORB.h>
#import "NamingContextProto.h"

@implementation HelloWorld

- init
{
    // 1. Get interface object
    id interface = [[ORB defaultORB]interfaceNamed:@"HelloWorld/HelloWorldIF"];
    if(interface){
        self = [super init];
        // 2. Create object with this interface
        object = [[CORBAObject allocWithZone:[ORB zone]]initWithInterface:interface];
        //object = [[ORB defaultORB]createObjectWithInterface:interface];
        // 3. Register implementation (self) 
        [object registerImplementation:self withKey:@"hello"];
        return self;
    }else
        NSLog(@"Cannot locate interface for 'HelloWorld/HelloWorldIF'\n");
    [self release];
    return nil;
}

- (void) run
{
    // 4. Get object's IOR
    NSString* ior = [object toString];
    NSString* path = [[NSFileManager defaultManager]currentDirectoryPath];
    CORBAObject* nameService = [ORB resolveInitialReferences:@"NameService"];
    // 5. Bind self in Naming Service, if provided
    if(nameService)
        [nameService rebind:[IDLNameComponent nameComponentsForPath:@"Hello.world"] object:object];
    // 6. Save IOR to the file
    if([ior writeToFile:[path stringByAppendingPathComponent:@"HWServer.ref"]
             atomically:YES]){
        NSLog(@"Saved IOR in './HWServer.ref'\n===> ORB is running ...\n");
        // 7. Run ORB
        [ORB run];
    }else
        NSLog(@"Cannot write IOR to file './HWServer.ref'\n");
}

// This class implements two functions 'hello_world'
- (NSString*) helloWorld
{
    IDLDefinition <IDLTypedValue> *msg = [[ORB defaultORB]definitionNamed:@"HelloWorld/Message"];
    return [msg value];
}
- (NSString*) helloWorld2
{
    IDLDefinition <IDLTypedValue>* msg = [[ORB defaultORB]definitionNamed:@"HelloWorld/Message2"];
    return [msg value];
}


@end
