#! /bin/sh

# The framework ADORB.framework must be copied to /Library/Frameworks or ~/Library/Frameworks
# for this example to run
# A symbolic link to existing ADORB.framework can be added instead.

if [ ! -r /Library/Frameworks/ADORB.framework -a ! -r ~/Library/Frameworks/ADORB.framework ]
then
echo Cannot locate ADORB.framework
echo Exiting ...
exit 1
fi


host=`hostname`
keep=N
if [ "X$1" = "X-keep" ]
then
keep=Y
shift
fi

port=${1:-15600}

# Other args can be:
# -trace YES				- traces remote calls
# -defaultIIOPVersion 1.2	- sets default GIOP version, default is 1.2
# -debug <options>			- prints debug information for remote calls, options is a combination of s, S, r, R

echo Using port $port 

# Start Naming Service orbd
# Comment next two lines if you run orbd manually with the port $port

orbd -ORBInitialPort $port &
sleep 5

# 1. Run 'NamingTest' server and bind it in the NamingService
./IDLTest -class DemoServer -ORBNamingURL corbaloc:iiop:1.2@localhost:$port -ORBidl demo.idl $*&
pid="$!"
sleep 1

# 2. Run 'NamingTest' client that acesses the server via NamingService #1
echo ------ Running DemoClient ------
./IDLTest -class DemoClient -ORBNamingURL corbaloc:iiop:1.2@localhost:$port -ORBidl demo.idl $*
sleep 1

# 3. Frame test
echo ------ Running ORBTest server ------
./IDLTest -class ORBTest -ORBidl ORBTest.idl -ORBNamingURL corbaloc:iiop:1.2@localhost:$port $*&
pid="$pid $!"
sleep 1
echo ------ Running ORBTest client ------
./IDLTest -class ORBTestClient  -ORBidl ORBTest.idl -port $port -useNSExt YES $*

# 4. Just don't break the tradition
echo ------ Running HelloWorld server ------
./IDLTest -class HelloWorld -ORBNamingURL corbaloc:iiop:1.2@localhost:$port -ORBidl ORBTest.idl $*&
pid="$pid $!"
sleep 1

echo ------ Running HelloWorld client ------
./IDLTest -class HelloWorldClient -ORBidl ORBTest.idl -ior HWServer.ref $*

if [ "$keep" = "N" ]
then
kill $pid
fi
