/*
 *  AdORB
 *  Copyright (C) 2006 Victor Ananiev
 *
 *  This file is part of ADORB.
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Library General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Library General Public License for more details.
 *
 *  You should have received a copy of the GNU Library General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#import "IDLSwitchCase.h"
#import "IDLDefinition_Private.h"
#import "IDLTypedValue.h"

@implementation IDLSwitchCase

- initWithCase:(NSNumber*)value dataType:(IDLDefinition*)type 
    switchType:(IDLDefinition*)obj name:(NSString*)name
{
    self = [super init];
    _switchType = [obj retain];
    _type = [type retain];
    _value = [value retain];
    _name = [name retain];
    return self;
}

+ switchCase:(NSNumber*)value dataType:(IDLDefinition*)type 
    switchType:(IDLDefinition*)obj name:(NSString*)name
{
    return [[[self allocWithZone:[self zone]]initWithCase:value dataType:type switchType:obj name:name]autorelease];
}

- (void) dealloc
{
    [_value release];
    [_type release];
    [_switchType release];
    [_name release];
    [super dealloc];
}

- (NSString*) description
{
    if(!_value)
        return [[super description] stringByAppendingFormat:@"\n<default>: %@ %@",
                _type, _name];
    else{
        NSString* val = [self caseStringValue];
        return [[super description] stringByAppendingFormat:@"\ncase %@: %@ %@",
                val,_type, _name];
    }
}

- (BOOL) isDefault;
{
    return _value == nil;
}

- (NSString*) name
{
    return _name;
}

- (NSString*) caseStringValue
{
    if(_value)
        return [_switchType valueDescriptionOf:_value];
    else
        return @"default";
}


- (IDLDefinition*) switchType
{
    return _switchType;
}

- caseValue
{
    return _value;
}

- (IDLDefinition*) caseType
{
    return _type;
}


- (void) encodeWithCORBACoder:(id <CORBACoder>)coder
{
    [coder encodeLong:[_value longValue]];
    [coder encodeString:[self name]];
    [_type encodeWithCORBACoder:coder];
}


- (void) encodeWithCoder:(NSCoder*)coder
{
	[super encodeWithCoder:coder];
	if([coder allowsKeyedCoding]){
		[coder encodeObject:_switchType forKey:@"switchType"];
		[coder encodeObject:_value forKey:@"value"];
		[coder encodeObject:_type forKey:@"type"];
		[coder encodeObject:_name forKey:@"name"];
	}else{
		[coder encodeObject:_switchType];
		[coder encodeObject:_value];
		[coder encodeObject:_type];
		[coder encodeObject:_name];
	}
}

- initWithCoder:(NSCoder*)coder
{
	self = [super initWithCoder:coder];
	if([coder allowsKeyedCoding]){
		_switchType = [[coder decodeObjectForKey:@"switchType"]retain];
		_value = [[coder decodeObjectForKey:@"value"]retain];
		_type = [[coder decodeObjectForKey:@"type"]retain];
		_name = [[coder decodeObjectForKey:@"name"]retain];
	}else{
		_switchType = [[coder decodeObject]retain];
		_value = [[coder decodeObject]retain];
		_type = [[coder decodeObject]retain];
		_name = [[coder decodeObject]retain];
	}
	return self;
}

@end
