/*
 *  AdORB
 *  Copyright (C) 2006 Victor Ananiev
 *
 *  This file is part of ADORB.
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Library General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Library General Public License for more details.
 *
 *  You should have received a copy of the GNU Library General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#import "IDLValueMember.h"
#import "IDLValueType_Private.h"
#import "IDLValueClass.h"
#import "IDLDefinition_Private.h"

@implementation IDLValueMember

- initWithType:(IDLDefinition*)type mode:(int)mode name:(NSString*)name
{
    self = [super init];
    _type = [type retain];
    _name = [name retain];
    _accessMode = mode;
    return self;
}

- (void) dealloc
{
    [_type release];
    [_name release];
    [super dealloc];
}

+ (IDLValueMember*)valueMemberOfType:(IDLDefinition*)type mode:(int)mode name:(NSString*)name
{
    return [[[self alloc]initWithType:type mode:mode name:name]autorelease];
}

- (NSString*) name
{
    return _name;
}

- (IDLDefinition*) memberType
{
    return _type;
}

- (short) accessMode
{
    return _accessMode;
}

- (void) encodeWithCoder:(NSCoder*)coder
{
	[super encodeWithCoder:coder];
	if([coder allowsKeyedCoding]){
		[coder encodeObject:_type forKey:@"type"];
		[coder encodeObject:_name forKey:@"name"];
		[coder encodeInt:_accessMode forKey:@"access"];
	}else{
		[coder encodeObject:_type];
		[coder encodeObject:_name];
		[coder encodeValueOfObjCType:@encode(short) at:&_accessMode];
	}
}

- initWithCoder:(NSCoder*)coder
{
	self = [super initWithCoder:coder];
	if([coder allowsKeyedCoding]){
		_type = [[coder decodeObjectForKey:@"type"]retain];
		_name = [[coder decodeObjectForKey:@"name"]retain];
		_accessMode = [coder decodeIntForKey:@"access"];
	}else{
		_type = [[coder decodeObject]retain];
		_name = [[coder decodeObject]retain];
		[coder decodeValueOfObjCType:@encode(short) at:&_accessMode];
	}
	return self;
}

@end
