/*
 *  AdORB
 *  Copyright (C) 2006 Victor Ananiev
 *
 *  This file is part of IDLBrowser.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

#import "OctetSequenceFormatter.h"


@implementation OctetSequenceFormatter

static OctetSequenceFormatter* _instance = nil;
static char* hex = "0123456789abcdef";

- initWithCount:(unsigned)count
{
    self = [super init];
    _count = count;
    return self;
}

+ formatterWithCount:(unsigned)count
{
    if(count > 0)
        return [[[self alloc]initWithCount:count]autorelease];
    else{
        if(!_instance)
            _instance = [[self alloc]initWithCount:0];
        return _instance;
    }
}

+ formatter
{
    return [self formatterWithCount:0];
}

- (NSString *)editingStringForObjectValue:data
{
    if([data isKindOfClass:[NSData class]]){
        NSString* value;
        int l = [data length];
        const unsigned char* b = [data bytes];
        char* str = NSZoneMalloc([self zone],2*l+1);
        char* s = str;
        while(l-- > 0){
            *s++ = hex[(*b)>>4];
            *s++ = hex[(*b)%16];
            b+=1;
        }
        *s++ = 0;
        value = [NSString stringWithUTF8String:str];
        NSZoneFree([self zone],str);
        return value;
    }else
        return nil;
}

static int hexToInt(unichar c)
{
    if(c>='0' && c<= '9')
        return c-'0';
    else
    if(c>= 'a' && c<='f')
        return c-'a'+10;
    else
    if(c>='A' && c<='F')
        return c-'A'+10;
    else
        return -1;
}

- (BOOL)getObjectValue:(id *)data forString:(NSString *)string errorDescription:(NSString **)error
{
    int l = [string length];
    if(l%2 != 0){
        if(error)
            *error = @"Not even number of HEX digits";
        return NO;
    }else
    if(l == 0){
        *data = [NSData data];
        return YES;
    }else{
        unsigned char* buf, *b;
        int c0, c1;
        int len = l/2;
		int k;
        b = buf = NSZoneMalloc([self zone],l);
        for(k=0;k<l;k+=2){
            if((c0 = hexToInt([string characterAtIndex:k])) < 0) break;
            if((c1 = hexToInt([string characterAtIndex:k+1])) < 0) break;
            *b++ = c0*16 + c1;
        }
        if(k < l){
            NSZoneFree([self zone],buf);
            if(error)
                *error = @"Not a HEX didgit in octet sequence";
            return NO;
        }else{
            *data = [NSData dataWithBytes:buf length:len];
            NSZoneFree([self zone],buf);
            return YES;
        }
    }
}

- (BOOL)isPartialStringValid:(NSString *)partialString 
        newEditingString:(NSString **)newString errorDescription:(NSString **)error
{
    int l = [partialString length];
    if(_count == 0 || l <= 2*_count){
		int k;
        for(k=0;k<l;k++){
			unichar c = [partialString characterAtIndex:k];
            if(!((c >='0' && c <= '9') ||
					(c >= 'a' && c <= 'f') ||
					(c >= 'A' && c <= 'F')))
				break;
		}
		if(k<l){
			*newString = [partialString substringToIndex:k];
			if(error)
				*error = @"Not a HEX digit in octet sequence";
                return NO;
        }
        return YES;
    }else{
        *newString = [partialString substringToIndex:2*_count];
        if(error)
            *error = @"Length exceeds maximum in octet sequence";
        return NO;
    }
}

- (NSString *) _stringForObjectValue:(NSData*)data
{
    NSString* value;
    int l = [data length];
    const unsigned char* b = [data bytes];
    char* str = NSZoneMalloc([self zone],2*l+3);
    char* s = str;
    *s++ = '0';
    *s++ = 'x';
    while(l-- > 0){
        *s++ = hex[(*b)>>4];
        *s++ = hex[(*b)%16];
        b+=1;
    }
    *s++ = 0;
    value = [NSString stringWithUTF8String:str];
    NSZoneFree([self zone],str);
    return value;
}

- (NSString *)stringForObjectValue:data
{
    if([data isKindOfClass:[NSData class]]){
        if([data length] > 0)
            return [self _stringForObjectValue:data];
        else
            return @"";
    }else
        return nil;
}
@end
